package org.greenstone.gatherer.util;

import java.io.Reader;
import java.io.IOException;

public class RemoveContentBeforeRootElementXMLReader extends Reader {

	static final byte[] xmlIndicator = "<?xml".getBytes();

	Reader ur;
	int foundBytesReturned = 0;
	boolean allFoundBytesReturned = false; //redundant, but may help performance


	public RemoveContentBeforeRootElementXMLReader( Reader ur ) {

		this.ur = ur;

		//read up to the xml indicator
		int foundBytes = 0;
		int c = 0;
		while ( c != -1 ) {
			try {
			    c = ur.read();
			} catch( Exception e ) {
				System.err.println( "Exception while reading underlying Reader in RemoveContentBeforeRootElementXMLReader" );
			}
			
			if ( c == (int)xmlIndicator[foundBytes] ) {
				foundBytes++;
			} else {			    
			    if ( foundBytes != 0 ) {
				for ( int i=0; i<foundBytes; i++ ) {
				    System.out.print( (char)xmlIndicator[i] );
				}
				foundBytes = 0;
			    }
			    if ( c != -1 ) {
				System.out.print( (char)c );
			    }
			}

			if ( foundBytes == xmlIndicator.length ) {
			    return;
			}

		}
		
		System.err.println( "RemoveContentBeforeRootElementXMLReader:\n" +
			"The XML being loaded did not contain the '<?xml' string as expected" );
	}

	public int read( char[] cbuf, int off, int len ) throws IOException {

		for ( int i=off; i<off+len && i<cbuf.length; i++ ) {

			//read from underlying reader
			int c = read();

			//catch end of stream
			if ( c == -1 ) {
				if ( i == off ) {
					return -1;
				}
				return i - off;
			}

			//insert character into the array
			cbuf[i] = (char)c;
		}
		return len;

	}

	public int read() throws IOException {

		if ( allFoundBytesReturned ) {
			return ur.read();
		}

		if ( foundBytesReturned < xmlIndicator.length ) {
			return xmlIndicator[foundBytesReturned++];
		}

		allFoundBytesReturned = true;
		return ur.read();
	}

	public void close() throws IOException {
		ur.close();
	}
}
